###############################################################################
#POD Part
###############################################################################

=head1 NAME

package I<Help> - library used to give all the help needed for user(s)

=head1 LIBRARY ORGANIZATION AND FUNCTIONS

	use Help;
	HelpDB();
	HelpClust();
	HelpDetailsDB();
	HelpDetailsClust();

=head1 DESCRIPTION

This module was developed to describe all programs, their usage and their options for the user(s).
The documentation was realized with B<Pod>.

=head1 CONSTRUCTOR

All functions take no argument as input. All functions return printed text to explain the functioning
of all programs and associated options. These functions will be automatically called when the programs
will encounter an error, or when the user chose the option I<-h>.

=head2 FUNCTIONS

=over 4

=item B<HelpDB()>

Print some help when the program Database_Format did not work.

=item B<HelpClust()>

Print some help when the program Database_Clustering did not work.

=item B<HelpDetailsDB()>

Print the complete help for the Database_Format program.

=item B<HelpDetailsClust()>

Print the complete help for the Database_Clustering program.

=back

=head1 AUTHORS

Sebastien TERRAT <sebastien.terrat@inra.fr>
Corentin Journay

=head1 TEAM AND NOTES

UMR 1347 INRA/uB/AgroSup - BioCom Team
RELEASED:  30/10/2018

=cut

###############################################################################
#FUNCTIONS AND PROGRAMS
###############################################################################

# Name of the package defined for the main program
package PerlLib::Help;
use strict;
use warnings;

###############################################################################
#FUNCTION:		HelpDB
#USAGE:				HelpDB()
#ARGUMENTS:		None
#DETAILS:			Print some help when the program did not work
#RELEASED: 05/10/2018
###############################################################################
sub HelpDB
{
	print "\n# Database Definition Program - RECLUSTOR 1.0 (December 2018)
Incorrect number of command line arguments.
Usage: ReClustOR_db.pl [-options] <filename>

With mandatory options are:
  -clust: clustering filename needed to create the database
  -taxo : taxonomy filename needed to create the database
  -derep: dereplication information filemane to create the database
  -seq  : FASTA file containing the sequences analyzed

To see more help on other available options, do ReClustOR_db.pl -h\n";
}
#End of function
###############################################################################

###############################################################################
#FUNCTION:		HelpClust
#USAGE:				HelpClust()
#ARGUMENTS:		None
#DETAILS:			Print some help when the program did not work
#RELEASED: 05/10/2018
###############################################################################
sub HelpClust
{
	print "# Clustering Program Using a Defined Database - RECLUSTOR 1.0 (December 2018)
# Copyright (C) 2018 INRA/Burgundy University - UMR 1347 Agroecology
# BioCom Team - Sebastien Terrat and Corentin Journay
# Freely distributed under the GNU General Public License (GPLv3)
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Usage: ReClustOR.pl [-options] <argument> <filename>

With mandatory options are:
  -db: Database folder containing the database definition files
  -th: Threshold chosen by the user for the comparison with the database
  -cl: Chosen clustering type [infern/needle]
  -sq: input FASTA file that will be compared to the database

By default, the program will be based on a comparison of input sequences against
the database given as a mandatory option (-db). The user must also give a
specific threshold level for the clustering step to avoid the selection of too
many results.
The clustering type is also mandatory. Two choices are available for the user:
 - infern: based on a global alignment of all reads against the database using
the INFERNAL program.
 - needle: using also a global alignment of reads, but with an algorithm closed
to the Needleman-Wunsch Algorithm.

To see more help on other available options, do ReClustOR.pl -h\n";
}
#End of function
#################################################################################

################################################################################
#FUNCTION:		HelpDetailsDB
#USAGE:				HelpDetailsDB()
#ARGUMENTS:		None
#DETAILS:			Print the complete help with the -h option
#RELEASED: 05/10/2018
################################################################################
sub HelpDetailsDB
{
	print "# Database Definition Program
# RECLUSTOR 1.0 (December 2018)
# Copyright (C) 2018 INRA/Burgundy University - UMR 1347 Agroecology
# BioCom Team - Sebastien Terrat and Corentin Journay
# Freely distributed under the GNU General Public License (GPLv3)
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Usage: ReClustOR_db.pl [-options] <filename>

With mandatory options are:
  -clust    : clustering filename needed to create the database
  -taxo     : taxonomy filename needed to create the database
  -derep    : dereplication information filename to create the database
  -seq      : FASTA file containing the sequences analyzed

# Read structure:
  All sequence Identifications (ID) must have a specific format to be accepted
by the Database Definition Program. The format is simple. Each sequence have to
be defined by an ID, the number of dereplicated reads and its length, all
elements separated by '_' characters as shown below.
Example: 	M0098755_6_length=374
This sequence had the ID: M0098755 (specific length not defined, but composed
only with alphanumeric characters (a-z,A-Z,0-9).
This sequence corresponded to 6 reads in the dataset, with a length of 374 bases.

The sequence must be also affiliated to a specific sample ID added to the name
(separated by a '|' character):
Example:	00001|M0098755_6_length=374
Here, this sequence M0098755_6_length=374 came from the sample 00001. The ID
for a sample must be a number only (0-9). Its length is not defined.

# Clustering file details:
  The clustering file had a specific format to be loaded by the PERL program.
Each line contains four elements, separated by tabulations. The line starts by
the OTU number (1 or OTU1), then the filename, then the number of reads in the
OTU and the IDs of all reads associated within this OTU, separated by a space.
Example:
1	filename	12	00003|M0098755_6_length=374 00001|M0098758_6_length=374
2	filename	5	00002|HB018GD8_5_length=371

It is important to highlight that the seed of the OTU (the OTU representative
sequence) is the first one in the list of each OTU. So this sequence must be
available in the FASTA file given in input.

# Taxonomic file details:
  Regarding the taxonomic file, it must have also a specific format to be used
by the program. For each OTU, five taxonomic levels must be defined (phylum,
class, order,family and genus), separated by a tabulation. For each taxonomic
level, one name must be given, with or without the number of corresponding
sequences affiliated in this OTU in parenthesis. The OTU number must be the
same for the taxonomic file and the clustering file to link both files.
Example:
OTU32	Actinobacteria(70)	Acidimicrobiia(70)	Acidimicrobiales(70)	Iamiaceae(70)	Iamia(70)

It is also possible to give several names for a taxonomic level, separated by
a space (see example for the genus level below). In the case where several
names are available, the number of sequences associated to this taxonomy
must be given to select the highest one automatically.
Example:
OTU32	Actinobacteria(70)	Acidimicrobiia(70)	Acidimicrobiales(70)	Iamiaceae(70)	Unknown(20) Iamia(50)

  Finally, all not affiliated groups must be called 'Unknown', to be clearly
identified by the Database Definition Program. All other names will be
considered as known organisms.

# Dereplication file details:
  This file is used to detail the composition of dereplicated sequences
encompassing identical sequences from several samples. To define the database
properly, the program must know the detailed representativity of all samples
in all OTUs. The specific format of this file is simple, as it is composed of
lines starting with the sequence ID encompassing the sequences from several
samples. The detail is given in the second part of the line, with each sample
sequence separated by a space. Both parts of the line are separated by a
tabulation.
Example:
00003|M0098755_6_length=374	00003|M0098755_3_length=374 00001|M0012749_3_length=374
Here, this sequence M0098755_6_length=374 encompass three sequences from the
sample 00003 and three from the 00001. In the clustering file, only the ID
sequence 00003|M0098755_6_length=374 will be found.

# Dependencies:
  This RECLUSTOR program includes two third-party programs: the Infernal alignment
tool (1.1.1 or higher) and the Inline PERL module (0.80 or higher). By default,
the program will check if these two mandatory tools are already installed or not.
If it is not the case, these two programs are required, and must be installed by
you.

# INFERNAL alignment:
  Infernal ('INFERence of RNA ALignment') is for searching DNA sequence
databases for RNA structure and sequence similarities. It is an implementation
of a special case of profile stochastic context-free grammars called covariance
models (CMs). A CM is like a sequence profile, but it scores a combination of
sequence consensus and RNA secondary structure consensus, so in many cases, it
is more capable of identifying RNA homologs that conserve their secondary
structure more than their primary sequence. You have to install also the
esl-alimerge program that allow the merging of alignments based on their
reference (RF) annotation (Easel i1.1.1 (July 2014), Copyright (C) 2014 Howard
Hughes Medical Institute, Freely distributed under the Janelia Farm Software
License).

Details and Download: http://eddylab.org/infernal/

# Inline module:
  The Inline module allows you to put source code from other programming
languages directly 'inline' in a Perl script or module. The code is
automatically compiled as needed, and then loaded for immediate access from
Perl.

Inline saves you from the hassle of having to write and compile your own
glue code using facilities like XS or SWIG. Simply type the code where you
want it and run your Perl as normal. All the hairy details are handled for
you. The compilation and installation of your code chunks all happen
transparently; all you will notice is the delay of compilation on the first
run.

The Inline code only gets compiled the first time you run it (or whenever it
is modified) so you only take the performance hit once. Code that is Inlined
into distributed modules (like on the CPAN) will get compiled when the module
is installed, so the end user will never notice the compilation time.

Details and Download: http://search.cpan.org/~ingy/Inline-0.80/lib/Inline.pod\n";
}
#End of function
###############################################################################

###############################################################################
#FUNCTION:		HelpDetailsClust
#USAGE:				HelpDetailsClust()
#ARGUMENTS:		None
#DETAILS:			Print the complete help with the -h option
#RELEASED: 05/10/2018
###############################################################################
sub HelpDetailsClust
{
	&HelpClust();
	print "

# Read structure:
  All sequence Identifications (ID) must have a specific format to be accepted
by the Database Definition Program and the Clustering Program. The format is
simple. Each sequence have to be defined by an ID, the number of dereplicated
reads and its length, all elements separated by '_' characters as shown below.
Example: 	M0098755_6_length=374
This sequence had the ID: M0098755 (specific length not defined, but composed
only with alphanumeric characters (a-z,A-Z,0-9).
This sequence corresponded to 6 reads in the dataset, with a length of 374
bases.

# Dependencies:
  This RECLUSTOR program includes two third-party programs: the Infernal alignment
tool (1.1.1 or higher) and the Inline PERL module (0.80 or higher). By default,
the program will check if these two mandatory tools are already installed or not.
If it is not the case, these two programs are required, and must be installed by
you.

# INFERNAL alignment:
  Infernal ('INFERence of RNA ALignment') is for searching DNA sequence
databases for RNA structure and sequence similarities. It is an implementation
of a special case of profile stochastic context-free grammars called covariance
models (CMs). A CM is like a sequence profile, but it scores a combination of
sequence consensus and RNA secondary structure consensus, so in many cases, it
is more capable of identifying RNA homologs that conserve their secondary
structure more than their primary sequence.
You have to install also the esl-alimerge program that allow the merging of
alignments based on their reference (RF) annotation (Easel i1.1.1 (July 2014),
Copyright (C) 2014 Howard Hughes Medical Institute, Freely distributed under the
Janelia Farm Software License).

Details and Download: http://eddylab.org/infernal/

# Inline module:
  The Inline module allows you to put source code from other programming
languages directly 'inline' in a Perl script or module. The code is
automatically compiled as needed, and then loaded for immediate access from
Perl.

Inline saves you from the hassle of having to write and compile your own
glue code using facilities like XS or SWIG. Simply type the code where you
want it and run your Perl as normal. All the hairy details are handled for
you. The compilation and installation of your code chunks all happen
transparently; all you will notice is the delay of compilation on the first
run.

The Inline code only gets compiled the first time you run it (or whenever it
is modified) so you only take the performance hit once. Code that is Inlined
into distributed modules (like on the CPAN) will get compiled when the module
is installed, so the end user will never notice the compilation time.

Details and Download: http://search.cpan.org/~ingy/Inline-0.80/lib/Inline.pod

# Additional options:

computing and parallelization analysis:
 -cp <i> : Number of cpus that the program will be able to use (default: 1)

optional output filename:
 -o <f>  : Results will be written in the <f> folder and not in the Results folder

optional clustering parameters:
 -c      : ignoring uncommon parts of compared reads at both extremities (default: not activated)
 -h      : ignoring homopolymer errors between reads (default: not activated)
 -hc     : ignoring both uncommon parts of compared reads at both extremities and
 homopolymer errors between reads (default: not activated)
 -ad     : add sequences with no similarities (at the defined threshold) against
 the database as new OTUs of the database. More precisely, the sequences will be
 integrated to the chosen database (-db option) to enrich it.
 BE CAREFUL WHEN USING THIS OPTION, AS THE DATABASE WILL BE MODIFIED, BUT WILL
 KEEP ITS INITIAL NAME!\n";
}
#End of function
###############################################################################
1;
