#!/usr/bin/perl

use strict;
use warnings;
#Needed libraries
use FindBin;
use lib $FindBin::RealBin;
use PerlLib::Help;
use PerlLib::CheckChoicesFolders;
use PerlLib::CheckFormats;
use PerlLib::CFunctions;
use PerlLib::Alignment;
use PerlLib::Clustering;
use PerlLib::TreatDataForClust;
use PerlLib::Formatting;
use PerlLib::EnrichDB;

#Defined variables
my $ref = "";
my $time = time;
my %Input_args = ();
my ($ref_otu, $ref_sample, $ref_abundance, $ref_gaps, $ref_parallel_data) = ("","","","","");

#############################################################################
# MAIN PROGRAM
#############################################################################

#Check if the user ask for help
if(defined $ARGV[0] && $ARGV[0] eq "-h") { PerlLib::Help::HelpDetailsClust(); exit(0); }
#Here, we catch and check the existence of all given arguments in input
$ref = PerlLib::CheckChoicesFolders::TreatInputArgumentsClust();
#Then, we will check all files based on their format, if we are good, then, we can start the treatment of the database
PerlLib::CheckFormats::CheckFileFormats($ref);
print (time-$time." seconds of treatment"."\n");
%Input_args = %{$ref};
#Here, we launch the treatment and/or creation of temporary and result folders
PerlLib::CheckChoicesFolders::CleanFolders($Input_args{"-o"});
#Here, we first launch the treatment of the dataset using either an alignment
#obtained with INFERNAL or a Needleman and Wunsch algorithm
($ref_otu, $ref_sample, $ref_abundance, $ref_gaps) = PerlLib::TreatDataForClust::ManagingInputData($ref);
#Then, here, we launched a function dedicated to the definition of sub-hashes to
#parallelize the analysis of the datasets.
$ref_parallel_data = PerlLib::TreatDataForClust::DefineParallelizedHashes($ref_abundance, $Input_args{"-cp"});
print ("END OF ORGANIZING DATA\n");
print (time-$time." Seconds of treatment"."\n");
#As we compare the sequences to the database, we launched the dedicated function to do this
print ("STARTING CLUSTERING DATA AGAINST THE DATABASE\n");
print (time-$time." Seconds of treatment"."\n");
PerlLib::Clustering::ClusteringAgainstDatabase(\%Input_args, $ref_otu, $ref_parallel_data, $ref_sample, $ref_gaps);
print ("END OF CLUSTERING DATA AGAINST THE DATABASE\n");
print (time-$time." Seconds of treatment"."\n");
#Here, we cleaned not needed variables to avoid problems of memory and simplify the
#treatment for further steps. Moreover, we launched the function dedicated to the selection of
($ref_sample, $ref_abundance) = PerlLib::Clustering::CleaningKeepingOuts($ref_sample, $ref_abundance);
#We check if needed to launch this step of DE NOVO clustering
if(keys(%{$ref_sample}) > 0)
{
	print ("STARTING CLUSTERING DATA USING A DE NOVO APPROACH\n");
	print (time-$time." Seconds of treatment"."\n");
	#Then, we launched the treatment of sequences that did not match against the database
	PerlLib::Clustering::ClusteringDeNovo(\%Input_args, $ref_sample, $ref_abundance);
	print ("END OF CLUSTERING DATA USING A DE NOVO APPROACH\n");
	print (time-$time." Seconds of treatment"."\n");
}
#Then, we format the results into several summary files for easy reading for users
print ("STARTING RESULTS FORMATTING\n");
print (time-$time." Seconds of treatment"."\n");
PerlLib::Formatting::FormattingResults($Input_args{"-sq"}, $Input_args{"-db"}, $Input_args{"-o"});
print ("ENDING RESULTS FORMATTING\n");
print (time-$time." Seconds of treatment"."\n");
#Finally, a last function is launched to add to the used database the new OTUs defined after
#a de novo approach.
if(exists($Input_args{"-ad"}))
{
	if(keys(%{$ref_sample}) > 0)
	{
		print ("STARTING DATABASE ENRICHMENT\n");
		print (time-$time." Seconds of treatment"."\n");
		PerlLib::EnrichDB::ManageEnrichDB($Input_args{"-sq"}, $Input_args{"-db"}, $Input_args{"-cp"}, $Input_args{"-o"});
		print ("ENDING DATABASE ENRICHMENT\n");
		print (time-$time." Seconds of treatment"."\n");
	}
	#We managed the special case that no reads was different from the database, if the user wants
	#an enrichment of its database
	else { print ("NO DATABASE ENRICHMENT DONE, ALL READS MATCHED TO THE DATABASE SEEDS\n"); }
}
exit(0);
