#!/usr/bin/perl


use strict;
use warnings;
#Needed libraries
use FindBin;
use lib $FindBin::RealBin;
use PerlLib::Help;
use PerlLib::CheckChoicesFolders;
use PerlLib::TreatDataForDb;
use PerlLib::CheckFormats;
use PerlLib::Alignment;


#Defined variables
my $length = 0;
my $time = time;
my $key = "";
my $ref = "";
my $ref_IDs = "";
my $ref_clust = "";
my $ref_corre = "";
my $ref_choices = "";
my $ref_glob_distrib = "";
my $ref_Taxo_IDs = "";
my $db_file = "";
my $db_folder = "";
my @ref_arrays = ();
my @choices = ();
my @seeds = ();
my %Input_args = ();
my %Final_data = ();
my %clust = ();
my %corre = ();

#############################################################################
# MAIN PROGRAM
#############################################################################

#Check if the user ask for help
if(defined $ARGV[0] && $ARGV[0] eq "-h") { PerlLib::Help::HelpDetailsDB(); exit(0); }
#Here, we catch and check the existence of all given files in input
$ref = PerlLib::CheckChoicesFolders::TreatInputArgumentsDB();
#Then, we will check all files based on their format, if we are good, then, we can start the treatment of the database
PerlLib::CheckFormats::CheckFileFormats($ref);
print (time-$time." Seconds of treatment"."\n");
%Input_args = %{$ref};
#Here, we catch all needed user choices to efficiently define the database name
$ref_choices = PerlLib::CheckChoicesFolders::CatchUserChoices();
print ("LAUNCHING THE EXTRACTION OF CLUSTERING DATA\n");
$db_folder = PerlLib::CheckChoicesFolders::DefineDatabaseDirectory($ref_choices);
#We then catch and treat the data from the Clustering and Dereplication files
@ref_arrays = PerlLib::TreatDataForDb::ExtractClustDerepData($ref_choices, \%Input_args);
$ref_IDs = $ref_arrays[3];
print ("END OF EXTRACTING CLUSTERING DATA\n");
print (time-$time." Seconds of treatment"."\n");
print ("LAUNCHING THE ANALYSIS OF THE CLUSTERING DATA\n");

@ref_arrays = PerlLib::TreatDataForDb::InvDerepClustData(@ref_arrays);
#We then store the needed variables for further analyses
$ref_corre = $ref_arrays[0];
$ref_clust = $ref_arrays[1];
print ("END OF ANALYSIS OF THE CLUSTERING DATA\n");
print (time-$time." Seconds of treatment"."\n");
print ("LAUNCHING THE ANALYSIS OF SAMPLE DISTRIBUTION\n");

#Here, we catch the details from the function that describe the composition of each OTU
$ref_glob_distrib = PerlLib::TreatDataForDb::DefineSamplingDistribution($ref_corre);
print ("END OF ANALYSIS OF SAMPLE DISTRIBUTION\n");
print (time-$time." Seconds of treatment"."\n");
print ("LAUNCHING THE DEFINITION OF OTU SEQUENCE FILE\n");

#Then, we catch the seeds IDs needed using @seeds, and we launch the dedicated function
#to create the dbreads file
($ref_IDs, $db_file) = PerlLib::TreatDataForDb::ExtractAndStoreSeedSeqs($Input_args{"-seq"}, $ref_IDs, $db_folder, $ref_choices);
print ("END OF DEFINITION OF OTU SEQUENCE FILE\n");
print (time-$time." Seconds of treatment"."\n");
print ("LAUNCHING THE ANALYSIS OF TAXONOMIC DATA\n");

#Here, we catch and extract for each OTU the taxonomic information needed
$ref_Taxo_IDs = PerlLib::TreatDataForDb::ExtractTaxonomicData($Input_args{"-taxo"});
print ("END OF ANALYSIS OF TAXONOMIC DATA\n");
print (time-$time." Seconds of treatment"."\n");
print ("LAUNCHING THE DATABASE DEFINITION FILES\n");

#Then, we define the database itself using all hashes and results from previous functions
PerlLib::TreatDataForDb::DefineDatabaseFile($ref_clust, $ref_glob_distrib, $ref_IDs, $ref_Taxo_IDs, $db_folder, $ref_choices);
print ("END OF DATABASE DEFINITION FILES\n");
print (time-$time." Seconds of treatment"."\n");
print ("LAUNCHING THE ALIGNMENT OF DATABASE SEQUENCES\n");

#Finally, we define the database itself using all hashes and results from previous functions
PerlLib::Alignment::LaunchInfernalAlign("$db_folder/$db_file", @{$ref_choices}[4], $Input_args{"-cp"}, "$db_folder/$db_file");
print ("END OF ALIGNMENT OF DATABASE SEQUENCES\n");
print (time-$time." Seconds of treatment"."\n");

#We finally quit the program itself
exit(0);
